from ctypes import c_char_p, c_uint64

from .frame import Frame, Topology
from .misc import ChemfilesError
from .utils import CxxPointer, _call_with_growing_buffer


class BaseTrajectory(CxxPointer):
    def __init__(self, ptr):
        self.__closed = False
        super(BaseTrajectory, self).__init__(ptr, is_const=False)

    def __check_opened(self):
        if self.__closed:
            raise ChemfilesError("Can not use a closed Trajectory")

    def __del__(self):
        if not self.__closed:
            self.close()

    def __enter__(self):
        self.__check_opened()
        return self

    def __exit__(self, *args):
        self.close()

    def __iter__(self):
        self.__check_opened()
        for step in range(self.nsteps):
            yield self.read_step(step)

    def read(self):
        """
        Read the next step of this :py:class:`Trajectory` and return the
        corresponding :py:class:`Frame`.
        """
        self.__check_opened()
        frame = Frame()
        self.ffi.chfl_trajectory_read(self.mut_ptr, frame.mut_ptr)
        return frame

    def read_step(self, step):
        """
        Read a specific ``step`` in this :py:class:`Trajectory` and return the
        corresponding :py:class:`Frame`.
        """
        self.__check_opened()
        frame = Frame()
        self.ffi.chfl_trajectory_read_step(self.mut_ptr, c_uint64(step), frame.mut_ptr)
        return frame

    def write(self, frame):
        """Write a :py:class:`Frame` to this :py:class:`Trajectory`."""
        self.__check_opened()
        self.ffi.chfl_trajectory_write(self.mut_ptr, frame.ptr)

    def set_topology(self, topology, format=""):
        """
        Set the :py:class:`Topology` associated with this :py:class:`Trajectory`.

        The new topology will be used when reading and writing the files,
        replacing any topology in the frames or files.

        If the ``topology`` parameter is a :py:class:`Topology` instance, it is
        used directly. If the ``topology`` parameter is a string, the first
        :py:class:`Frame` of the corresponding file is read, and the topology of
        this frame is used.

        When reading from a file, if ``format`` is not the empty string, it is
        used as the file format instead of guessing it from the file extension.
        """
        self.__check_opened()
        if isinstance(topology, Topology):
            self.ffi.chfl_trajectory_set_topology(self.mut_ptr, topology.ptr)
        else:
            self.ffi.chfl_trajectory_topology_file(
                self.mut_ptr, topology.encode("utf8"), format.encode("utf8")
            )

    def set_cell(self, cell):
        """
        Set the :py:class:`UnitCell` associated with this :py:class:`Trajectory`
        to a copy of ``cell``.

        This :py:class:`UnitCell` will be used when reading and writing the
        files, replacing any unit cell in the frames or files.
        """
        self.__check_opened()
        self.ffi.chfl_trajectory_set_cell(self.mut_ptr, cell.ptr)

    @property
    def nsteps(self):
        """Get the current number of steps in this :py:class:`Trajectory`."""
        self.__check_opened()
        nsteps = c_uint64()
        self.ffi.chfl_trajectory_nsteps(self.mut_ptr, nsteps)
        return nsteps.value

    @property
    def path(self):
        """Get the path used to open this :py:class:`Trajectory`."""
        self.__check_opened()
        return _call_with_growing_buffer(
            lambda buffer, size: self.ffi.chfl_trajectory_path(self.ptr, buffer, size),
            initial=256,
        )

    def close(self):
        """
        Close this :py:class:`Trajectory` and write any buffered content to the
        file.
        """
        self.__check_opened()
        self.__closed = True
        self.ffi.chfl_trajectory_close(self.ptr)


class Trajectory(BaseTrajectory):
    """
    A :py:class:`Trajectory` represent a physical file from which we can read
    :py:class:`Frame`.
    """

    def __init__(self, path, mode="r", format=""):
        """
        Open the file at the given ``path`` using the given ``mode`` and
        optional file ``format``.

        Valid modes are ``'r'`` for read, ``'w'`` for write and ``'a'`` for
        append.

        The ``format`` parameter is needed when the file format does not match
        the extension, or when there is not standard extension for this format.
        If `format` is an empty string, the format will be guessed from the
        file extension.
        """
        ptr = self.ffi.chfl_trajectory_with_format(
            path.encode("utf8"), mode.encode("utf8"), format.encode("utf8")
        )
        # Store mode and format for __repr__
        self.__mode = mode
        self.__format = format
        super(Trajectory, self).__init__(ptr)

    def __repr__(self):
        return f"Trajectory('{self.path}', '{self.__mode}', '{self.__format}')"


class MemoryTrajectory(BaseTrajectory):
    """
    A :py:class:`MemoryTrajectory` allow to read/write in-memory data as though
    it was a formatted file.
    """

    def __init__(self, data="", mode="r", format=""):
        """
        The ``format`` parameter is always required.

        When reading (``mode`` is ``'r'``), the ``data`` parameter will be used
        as the formatted file.

        When writing (``mode`` is ``'w'``), the ``data`` parameter is ignored.
        To get the memory buffer containing everything already written, use the
        :py:func:`buffer` function.
        """

        if not format:
            raise ChemfilesError(
                "'format' is required when creating a MemoryTrajectory"
            )

        if mode == "r":
            if isinstance(data, str):
                data = data.encode("utf8")
            elif not isinstance(data, bytes):
                raise ChemfilesError("the 'data' parameter must be a string")

            ptr = self.ffi.chfl_trajectory_memory_reader(
                data, len(data), format.encode("utf8")
            )
        elif mode == "w":
            ptr = self.ffi.chfl_trajectory_memory_writer(format.encode("utf8"))
        else:
            raise ChemfilesError(f"invalid mode '{mode}' passed to MemoryTrajectory")

        super(MemoryTrajectory, self).__init__(ptr)

    def __repr__(self):
        return f"MemoryTrajectory({self.__mode}', '{self.__format}')"

    def buffer(self):
        """
        Get the data written to this in-memory trajectory. This is not valid to
        call when reading in-memory data.
        """
        buffer = c_char_p()
        size = c_uint64()
        self.ffi.chfl_trajectory_memory_buffer(self.ptr, buffer, size)

        return buffer.value



#!/usr/bin/env python
# -*- coding: utf8 -*-
"""
Check that only chemfiles symbols are exported by the shared library.
"""
import os
import subprocess
import sys

ROOT = os.path.join(os.path.dirname(__file__), "..", "..")
ERRORS = 0


def error(message):
    global ERRORS
    ERRORS += 1
    print(message)


def list_symbols(path):
    list = subprocess.check_output(
        ["nm", "-D", "-C", path],
        stderr=subprocess.STDOUT,
    ).decode("utf8")
    symbols = []
    for line in list.split("\n"):
        # Only global code symbols
        if " T " in line:
            name = " ".join(line.split()[2:])
            symbols.append(name)
    return symbols


if __name__ == "__main__":
    for symbol in list_symbols(sys.argv[1]):
        if symbol.startswith("std::"):
            continue
        elif symbol.startswith("chemfiles::"):
            continue
        elif symbol.startswith("chfl_"):
            continue
        elif symbol.startswith("transaction clone"):
            continue
        elif symbol in ["_init", "_fini"]:
            continue
        else:
            error("External symbol {} is exported".format(symbol))

    if ERRORS != 0:
        sys.exit(1)



import warnings
from ctypes import POINTER, c_uint64, create_string_buffer

from ._c_lib import _get_c_library


class ChemfilesWarning(UserWarning):
    """Warnings from the Chemfiles runtime."""

    pass


class ChemfilesError(BaseException):
    """Exception class for errors in chemfiles"""

    pass


class FormatMetadata:
    """
    :py:class:`FormatMetadata` contains metadata associated with one format.
    The following fields are directly accessible:

    :param str name: name of the format
    :param str extension: extension associated with the format, or ``None`` if
                          there is no extension associated with this format
    :param str description: extended user-facing description of the format
    :param str reference: extended user-facing description of the format

    :param bool read: is reading files in this format implemented?
    :param bool write: is writing files in this format implemented?
    :param bool memory: does this format support in-memory IO?

    :param bool positions: does this format support storing atomic positions?
    :param bool velocities: does this format support storing atomic velocities?
    :param bool unit_cell: does this format support storing unit cell information?
    :param bool atoms: does this format support storing atom names or types?
    :param bool bonds: does this format support storing bonds between atoms?
    :param bool residues: does this format support storing residues?
    """

    def __init__(self):
        self.name = ""
        self.extension = None
        self.description = ""
        self.reference = ""

        self.read = False
        self.write = False
        self.memory = False

        self.positions = False
        self.velocities = False
        self.unit_cell = False
        self.atoms = False
        self.bonds = False
        self.residues = False

    def _set_from_c(self, c_format_metadata):
        self.name = c_format_metadata.name.decode("utf8")

        if c_format_metadata.extension is None:
            self.extension = None
        else:
            self.extension = c_format_metadata.extension.decode("utf8")

        self.description = c_format_metadata.description.decode("utf8")
        self.reference = c_format_metadata.reference.decode("utf8")

        self.read = c_format_metadata.read
        self.write = c_format_metadata.write
        self.memory = c_format_metadata.memory

        self.positions = c_format_metadata.positions
        self.velocities = c_format_metadata.velocities
        self.unit_cell = c_format_metadata.unit_cell
        self.atoms = c_format_metadata.atoms
        self.bonds = c_format_metadata.bonds
        self.residues = c_format_metadata.residues

    def __repr__(self):
        return f"""
FormatMetadata for {self.name}
-------------------{"-" * len(self.name)}
{self.description}

name      = {self.name}
extension = {self.extension}
reference = {self.reference}

Capacities:
-----------

read        = {self.read}
write       = {self.write}
memory      = {self.memory}
positions   = {self.positions}
velocities  = {self.velocities}
unit_cell   = {self.unit_cell}
atoms       = {self.atoms}
bonds       = {self.bonds}
residues    = {self.residues}
"""


def formats_list():
    """
    Get the list of formats known by chemfiles, as well as all associated
    metadata.

    :rtype: list(FormatMetadata)
    """
    from ._c_api import chfl_format_metadata

    lib = _get_c_library()

    array = POINTER(chfl_format_metadata)()
    count = c_uint64()
    lib.chfl_formats_list(array, count)

    formats = [FormatMetadata() for i in range(count.value)]
    for i in range(count.value):
        formats[i]._set_from_c(array[i])
    lib.chfl_free(array)

    return formats


# Store a reference to the last logging callback, to preven Python from
# garbage-collecting it.
_CURRENT_CALLBACK = None


def set_warnings_callback(function):
    """
    Call ``function`` on every warning event. The callback should take a string
    message and return nothing.

    By default, warnings are send to python ``warnings`` module.
    """
    from ._c_api import chfl_warning_callback

    def callback(message):
        try:
            function(message.decode("utf8"))
        except Exception as e:
            message = f"exception raised in warning callback: {e}"
            warnings.warn(message, ChemfilesWarning)

    global _CURRENT_CALLBACK
    _CURRENT_CALLBACK = chfl_warning_callback(callback)

    _get_c_library().chfl_set_warning_callback(_CURRENT_CALLBACK)


def add_configuration(path):
    """
    Read configuration data from the file at ``path``.

    By default, chemfiles reads configuration from any file named
    ``.chemfilesrc`` in the current directory or any parent directory. This
    function can be used to add data from another configuration file.

    This function will fail if there is no file at ``path``, or if the file is
    incorrectly formatted. Data from the new configuration file will overwrite
    any existing data.
    """
    _get_c_library().chfl_add_configuration(path.encode("utf8"))


def _last_error():
    """Get the last error from the chemfiles runtime."""
    return _get_c_library().chfl_last_error().decode("utf8")


def _clear_errors():
    """Clear any error message saved in the chemfiles runtime."""
    return _get_c_library().chfl_clear_errors()


def _set_default_warning_callback():
    set_warnings_callback(
        # We need to set stacklevel=4 to get through the lambda =>
        # adaptor => C++ code => Python binding => user code
        lambda message: warnings.warn(message, ChemfilesWarning, stacklevel=4)
    )


def guess_format(path):
    """
    Get the format that chemfiles would use to read a file at the given
    ``path``.

    The format is mostly guessed from the path extension, chemfiles only tries
    to read the file to distinguish between CIF and mmCIF files. Opening the
    file using the returned format string might still fail. For example, it will
    fail if the file is not actually formatted according to the guessed format;
    or the format/compression combination is not supported (e.g. ``XTC / GZ``
    will not work since the XTC reader does not support compressed files).

    The returned format is represented in a way compatible with the various
    ``Trajectory`` constructors, i.e. ``"<format name> [/ <compression>]"``,
    where compression is optional.
    """
    lib = _get_c_library()

    buffer = create_string_buffer(b"\0", 128)
    lib.chfl_guess_format(path.encode("utf8"), buffer, 128)
    return buffer.value.decode("utf8")



from .atom import Atom
from .cell import CellShape, UnitCell
from .frame import Frame
from .misc import (
    ChemfilesError,
    add_configuration,
    formats_list,
    guess_format,
    set_warnings_callback,
)
from .property import Property
from .residue import Residue
from .selection import Selection
from .topology import BondOrder, Topology
from .trajectory import MemoryTrajectory, Trajectory

__version__ = "0.10.4"



import os
import re
import site
import subprocess
import sys

from setuptools import Extension, setup
from setuptools.command.bdist_egg import bdist_egg
from setuptools.command.build_ext import build_ext
from setuptools.command.build_py import build_py
from wheel.bdist_wheel import bdist_wheel

# workaround https://github.com/pypa/pip/issues/7953
site.ENABLE_USER_SITE = "--user" in sys.argv[1:]

# Read the version from chemfiles/__init__.py without importing chemfiles
ROOT = os.path.realpath(os.path.dirname(__file__))
__version__ = re.search(
    r'__version__\s*=\s*[\'"]([^\'"]*)[\'"]', open("src/chemfiles/__init__.py").read()
).group(1)


class universal_wheel(bdist_wheel):
    # Workaround until https://github.com/pypa/wheel/issues/185 is resolved
    def get_tag(self):
        tag = bdist_wheel.get_tag(self)
        return ("py2.py3", "none") + tag[2:]


class cmake_configure(build_py):
    """configure cmake to build chemfiles and generate external.py"""

    def run(self):
        source_dir = ROOT
        build_dir = os.path.join(ROOT, "build", "cmake-build")
        install_dir = os.path.join(os.path.realpath(self.build_lib))

        try:
            os.makedirs(build_dir)
        except OSError:
            pass

        cmake_options = [
            "-GNinja",
            f"-DCMAKE_INSTALL_PREFIX={install_dir}",
            "-DCMAKE_BUILD_TYPE=Release",
            "-DBUILD_SHARED_LIBS=ON",
        ]

        if sys.platform.startswith("darwin"):
            cmake_options.append("-DCMAKE_OSX_DEPLOYMENT_TARGET:STRING=10.9")

        # ARCHFLAGS is used by cibuildwheel to pass the requested arch to the
        # compilers
        ARCHFLAGS = os.environ.get("ARCHFLAGS")
        if ARCHFLAGS is not None:
            cmake_options.append(f"-DCMAKE_C_FLAGS={ARCHFLAGS}")
            cmake_options.append(f"-DCMAKE_CXX_FLAGS={ARCHFLAGS}")

        if os.getenv("CHFL_PY_INTERNAL_CHEMFILES"):
            cmake_options.append("-DCHFL_PY_INTERNAL_CHEMFILES=ON")

        subprocess.run(
            ["cmake", source_dir, *cmake_options],
            cwd=build_dir,
            check=True,
        )

        return super().run()


class cmake_build(build_ext):
    """build and install chemfiles with cmake"""

    def run(self):
        build_dir = os.path.join(ROOT, "build", "cmake-build")

        subprocess.run(
            ["cmake", "--build", build_dir],
            check=True,
        )

        subprocess.run(
            ["cmake", "--install", build_dir, "--strip"],
            check=True,
        )


class bdist_egg_disabled(bdist_egg):
    """Disabled version of bdist_egg

    Prevents setup.py install performing setuptools' default easy_install,
    which it should never ever do.
    """

    def run(self):
        sys.exit(
            "Aborting implicit building of eggs. "
            + "Use `pip install .` or `python setup.py bdist_wheel && pip "
            + "uninstall chemfiles -y && pip install dist/chemfiles-*.whl` "
            + "to install from source."
        )


setup(
    version=__version__,
    ext_modules=[
        # only declare the extension, it is built & copied as required by cmake
        # in the build_ext command
        Extension(name="chemfiles", sources=[]),
    ],
    cmdclass={
        "build_py": cmake_configure,
        "build_ext": cmake_build,
        "bdist_wheel": universal_wheel,
        "bdist_egg": bdist_egg if "bdist_egg" in sys.argv else bdist_egg_disabled,
    },
    exclude_package_data={
        "chemfiles": [
            "include/*",
        ]
    },
)



# -* coding: utf-8 -*
# Chemfiles, a modern library for chemistry file reading and writing
# Copyright (C) Guillaume Fraux and contributors -- BSD license
#
# =========================================================================== #
# !!!! AUTO-GENERATED FILE !!!! Do not edit. See the bindgen repository for
# the generation code (https://github.com/chemfiles/bindgen).
# This file contains Python ctype interface to the C API
# =========================================================================== #

# flake8: noqa
'''
Foreign function interface declaration for the Python interface to chemfiles
'''
from numpy.ctypeslib import ndpointer
import numpy as np
from ctypes import c_int, c_int64, c_uint64, c_double, c_char, c_char_p, c_void_p, c_bool
from ctypes import CFUNCTYPE, ARRAY, POINTER, Structure

from .utils import _check_return_code


class chfl_status(c_int):
    CHFL_SUCCESS = 0
    CHFL_MEMORY_ERROR = 1
    CHFL_FILE_ERROR = 2
    CHFL_FORMAT_ERROR = 3
    CHFL_SELECTION_ERROR = 4
    CHFL_CONFIGURATION_ERROR = 5
    CHFL_OUT_OF_BOUNDS = 6
    CHFL_PROPERTY_ERROR = 7
    CHFL_GENERIC_ERROR = 254
    CHFL_CXX_ERROR = 255


class chfl_bond_order(c_int):
    CHFL_BOND_UNKNOWN = 0
    CHFL_BOND_SINGLE = 1
    CHFL_BOND_DOUBLE = 2
    CHFL_BOND_TRIPLE = 3
    CHFL_BOND_QUADRUPLE = 4
    CHFL_BOND_QUINTUPLET = 5
    CHFL_BOND_AMIDE = 254
    CHFL_BOND_AROMATIC = 255


class chfl_property_kind(c_int):
    CHFL_PROPERTY_BOOL = 0
    CHFL_PROPERTY_DOUBLE = 1
    CHFL_PROPERTY_STRING = 2
    CHFL_PROPERTY_VECTOR3D = 3


class chfl_cellshape(c_int):
    CHFL_CELL_ORTHORHOMBIC = 0
    CHFL_CELL_TRICLINIC = 1
    CHFL_CELL_INFINITE = 2


class CHFL_TRAJECTORY(Structure):
    pass


class CHFL_CELL(Structure):
    pass


class CHFL_ATOM(Structure):
    pass


class CHFL_FRAME(Structure):
    pass


class CHFL_TOPOLOGY(Structure):
    pass


class CHFL_SELECTION(Structure):
    pass


class CHFL_RESIDUE(Structure):
    pass


class CHFL_PROPERTY(Structure):
    pass

# Some hand-defined type. Make sure to edit the bindgen code to make this
# correspond to the current chemfiles.h header
chfl_vector3d = ARRAY(c_double, 3)

chfl_warning_callback = CFUNCTYPE(None, c_char_p)

class chfl_match(Structure):
    _fields_ = [
        ('size', c_uint64),
        ('atoms', ARRAY(c_uint64, 4))
    ]

class chfl_format_metadata(Structure):
    _fields_ = [
        ('name', c_char_p),
        ('extension', c_char_p),
        ('description', c_char_p),
        ('reference', c_char_p),

        ('read', c_bool),
        ('write', c_bool),
        ('memory', c_bool),
        ('positions', c_bool),
        ('velocities', c_bool),
        ('unit_cell', c_bool),
        ('atoms', c_bool),
        ('bonds', c_bool),
        ('residues', c_bool),
    ]

# end of hand-defined types


def set_interface(c_lib):
    # Manually defined functions
    c_lib.chfl_free.argtypes = [c_void_p]
    c_lib.chfl_trajectory_close.argtypes = [POINTER(CHFL_TRAJECTORY)]
    # End of manually defined functions

    # Function "chfl_version", at types.h:150
    c_lib.chfl_version.argtypes = []
    c_lib.chfl_version.restype = c_char_p

    # Function "chfl_last_error", at misc.h:23
    c_lib.chfl_last_error.argtypes = []
    c_lib.chfl_last_error.restype = c_char_p

    # Function "chfl_clear_errors", at misc.h:33
    c_lib.chfl_clear_errors.argtypes = []
    c_lib.chfl_clear_errors.restype = chfl_status
    c_lib.chfl_clear_errors.errcheck = _check_return_code

    # Function "chfl_set_warning_callback", at misc.h:42
    c_lib.chfl_set_warning_callback.argtypes = [chfl_warning_callback]
    c_lib.chfl_set_warning_callback.restype = chfl_status
    c_lib.chfl_set_warning_callback.errcheck = _check_return_code

    # Function "chfl_add_configuration", at misc.h:58
    c_lib.chfl_add_configuration.argtypes = [c_char_p]
    c_lib.chfl_add_configuration.restype = chfl_status
    c_lib.chfl_add_configuration.errcheck = _check_return_code

    # Function "chfl_formats_list", at misc.h:71
    c_lib.chfl_formats_list.argtypes = [POINTER(POINTER(chfl_format_metadata)), POINTER(c_uint64)]
    c_lib.chfl_formats_list.restype = chfl_status
    c_lib.chfl_formats_list.errcheck = _check_return_code

    # Function "chfl_guess_format", at misc.h:93
    c_lib.chfl_guess_format.argtypes = [c_char_p, c_char_p, c_uint64]
    c_lib.chfl_guess_format.restype = chfl_status
    c_lib.chfl_guess_format.errcheck = _check_return_code

    # Function "chfl_property_bool", at property.h:37
    c_lib.chfl_property_bool.argtypes = [c_bool]
    c_lib.chfl_property_bool.restype = POINTER(CHFL_PROPERTY)

    # Function "chfl_property_double", at property.h:47
    c_lib.chfl_property_double.argtypes = [c_double]
    c_lib.chfl_property_double.restype = POINTER(CHFL_PROPERTY)

    # Function "chfl_property_string", at property.h:57
    c_lib.chfl_property_string.argtypes = [c_char_p]
    c_lib.chfl_property_string.restype = POINTER(CHFL_PROPERTY)

    # Function "chfl_property_vector3d", at property.h:67
    c_lib.chfl_property_vector3d.argtypes = [chfl_vector3d]
    c_lib.chfl_property_vector3d.restype = POINTER(CHFL_PROPERTY)

    # Function "chfl_property_get_kind", at property.h:74
    c_lib.chfl_property_get_kind.argtypes = [POINTER(CHFL_PROPERTY), POINTER(chfl_property_kind)]
    c_lib.chfl_property_get_kind.restype = chfl_status
    c_lib.chfl_property_get_kind.errcheck = _check_return_code

    # Function "chfl_property_get_bool", at property.h:87
    c_lib.chfl_property_get_bool.argtypes = [POINTER(CHFL_PROPERTY), POINTER(c_bool)]
    c_lib.chfl_property_get_bool.restype = chfl_status
    c_lib.chfl_property_get_bool.errcheck = _check_return_code

    # Function "chfl_property_get_double", at property.h:100
    c_lib.chfl_property_get_double.argtypes = [POINTER(CHFL_PROPERTY), POINTER(c_double)]
    c_lib.chfl_property_get_double.restype = chfl_status
    c_lib.chfl_property_get_double.errcheck = _check_return_code

    # Function "chfl_property_get_string", at property.h:115
    c_lib.chfl_property_get_string.argtypes = [POINTER(CHFL_PROPERTY), c_char_p, c_uint64]
    c_lib.chfl_property_get_string.restype = chfl_status
    c_lib.chfl_property_get_string.errcheck = _check_return_code

    # Function "chfl_property_get_vector3d", at property.h:128
    c_lib.chfl_property_get_vector3d.argtypes = [POINTER(CHFL_PROPERTY), chfl_vector3d]
    c_lib.chfl_property_get_vector3d.restype = chfl_status
    c_lib.chfl_property_get_vector3d.errcheck = _check_return_code

    # Function "chfl_atom", at atom.h:24
    c_lib.chfl_atom.argtypes = [c_char_p]
    c_lib.chfl_atom.restype = POINTER(CHFL_ATOM)

    # Function "chfl_atom_copy", at atom.h:34
    c_lib.chfl_atom_copy.argtypes = [POINTER(CHFL_ATOM)]
    c_lib.chfl_atom_copy.restype = POINTER(CHFL_ATOM)

    # Function "chfl_atom_from_frame", at atom.h:60
    c_lib.chfl_atom_from_frame.argtypes = [POINTER(CHFL_FRAME), c_uint64]
    c_lib.chfl_atom_from_frame.restype = POINTER(CHFL_ATOM)

    # Function "chfl_atom_from_topology", at atom.h:83
    c_lib.chfl_atom_from_topology.argtypes = [POINTER(CHFL_TOPOLOGY), c_uint64]
    c_lib.chfl_atom_from_topology.restype = POINTER(CHFL_ATOM)

    # Function "chfl_atom_mass", at atom.h:94
    c_lib.chfl_atom_mass.argtypes = [POINTER(CHFL_ATOM), POINTER(c_double)]
    c_lib.chfl_atom_mass.restype = chfl_status
    c_lib.chfl_atom_mass.errcheck = _check_return_code

    # Function "chfl_atom_set_mass", at atom.h:103
    c_lib.chfl_atom_set_mass.argtypes = [POINTER(CHFL_ATOM), c_double]
    c_lib.chfl_atom_set_mass.restype = chfl_status
    c_lib.chfl_atom_set_mass.errcheck = _check_return_code

    # Function "chfl_atom_charge", at atom.h:112
    c_lib.chfl_atom_charge.argtypes = [POINTER(CHFL_ATOM), POINTER(c_double)]
    c_lib.chfl_atom_charge.restype = chfl_status
    c_lib.chfl_atom_charge.errcheck = _check_return_code

    # Function "chfl_atom_set_charge", at atom.h:121
    c_lib.chfl_atom_set_charge.argtypes = [POINTER(CHFL_ATOM), c_double]
    c_lib.chfl_atom_set_charge.restype = chfl_status
    c_lib.chfl_atom_set_charge.errcheck = _check_return_code

    # Function "chfl_atom_type", at atom.h:131
    c_lib.chfl_atom_type.argtypes = [POINTER(CHFL_ATOM), c_char_p, c_uint64]
    c_lib.chfl_atom_type.restype = chfl_status
    c_lib.chfl_atom_type.errcheck = _check_return_code

    # Function "chfl_atom_set_type", at atom.h:142
    c_lib.chfl_atom_set_type.argtypes = [POINTER(CHFL_ATOM), c_char_p]
    c_lib.chfl_atom_set_type.restype = chfl_status
    c_lib.chfl_atom_set_type.errcheck = _check_return_code

    # Function "chfl_atom_name", at atom.h:152
    c_lib.chfl_atom_name.argtypes = [POINTER(CHFL_ATOM), c_char_p, c_uint64]
    c_lib.chfl_atom_name.restype = chfl_status
    c_lib.chfl_atom_name.errcheck = _check_return_code

    # Function "chfl_atom_set_name", at atom.h:163
    c_lib.chfl_atom_set_name.argtypes = [POINTER(CHFL_ATOM), c_char_p]
    c_lib.chfl_atom_set_name.restype = chfl_status
    c_lib.chfl_atom_set_name.errcheck = _check_return_code

    # Function "chfl_atom_full_name", at atom.h:173
    c_lib.chfl_atom_full_name.argtypes = [POINTER(CHFL_ATOM), c_char_p, c_uint64]
    c_lib.chfl_atom_full_name.restype = chfl_status
    c_lib.chfl_atom_full_name.errcheck = _check_return_code

    # Function "chfl_atom_vdw_radius", at atom.h:185
    c_lib.chfl_atom_vdw_radius.argtypes = [POINTER(CHFL_ATOM), POINTER(c_double)]
    c_lib.chfl_atom_vdw_radius.restype = chfl_status
    c_lib.chfl_atom_vdw_radius.errcheck = _check_return_code

    # Function "chfl_atom_covalent_radius", at atom.h:195
    c_lib.chfl_atom_covalent_radius.argtypes = [POINTER(CHFL_ATOM), POINTER(c_double)]
    c_lib.chfl_atom_covalent_radius.restype = chfl_status
    c_lib.chfl_atom_covalent_radius.errcheck = _check_return_code

    # Function "chfl_atom_atomic_number", at atom.h:205
    c_lib.chfl_atom_atomic_number.argtypes = [POINTER(CHFL_ATOM), POINTER(c_uint64)]
    c_lib.chfl_atom_atomic_number.restype = chfl_status
    c_lib.chfl_atom_atomic_number.errcheck = _check_return_code

    # Function "chfl_atom_properties_count", at atom.h:212
    c_lib.chfl_atom_properties_count.argtypes = [POINTER(CHFL_ATOM), POINTER(c_uint64)]
    c_lib.chfl_atom_properties_count.restype = chfl_status
    c_lib.chfl_atom_properties_count.errcheck = _check_return_code

    # Function "chfl_atom_list_properties", at atom.h:228
    c_lib.chfl_atom_list_properties.argtypes = [POINTER(CHFL_ATOM), POINTER(c_char_p), c_uint64]
    c_lib.chfl_atom_list_properties.restype = chfl_status
    c_lib.chfl_atom_list_properties.errcheck = _check_return_code

    # Function "chfl_atom_set_property", at atom.h:240
    c_lib.chfl_atom_set_property.argtypes = [POINTER(CHFL_ATOM), c_char_p, POINTER(CHFL_PROPERTY)]
    c_lib.chfl_atom_set_property.restype = chfl_status
    c_lib.chfl_atom_set_property.errcheck = _check_return_code

    # Function "chfl_atom_get_property", at atom.h:254
    c_lib.chfl_atom_get_property.argtypes = [POINTER(CHFL_ATOM), c_char_p]
    c_lib.chfl_atom_get_property.restype = POINTER(CHFL_PROPERTY)

    # Function "chfl_residue", at residue.h:25
    c_lib.chfl_residue.argtypes = [c_char_p]
    c_lib.chfl_residue.restype = POINTER(CHFL_RESIDUE)

    # Function "chfl_residue_with_id", at residue.h:35
    c_lib.chfl_residue_with_id.argtypes = [c_char_p, c_int64]
    c_lib.chfl_residue_with_id.restype = POINTER(CHFL_RESIDUE)

    # Function "chfl_residue_from_topology", at residue.h:63
    c_lib.chfl_residue_from_topology.argtypes = [POINTER(CHFL_TOPOLOGY), c_uint64]
    c_lib.chfl_residue_from_topology.restype = POINTER(CHFL_RESIDUE)

    # Function "chfl_residue_for_atom", at residue.h:90
    c_lib.chfl_residue_for_atom.argtypes = [POINTER(CHFL_TOPOLOGY), c_uint64]
    c_lib.chfl_residue_for_atom.restype = POINTER(CHFL_RESIDUE)

    # Function "chfl_residue_copy", at residue.h:102
    c_lib.chfl_residue_copy.argtypes = [POINTER(CHFL_RESIDUE)]
    c_lib.chfl_residue_copy.restype = POINTER(CHFL_RESIDUE)

    # Function "chfl_residue_atoms_count", at residue.h:109
    c_lib.chfl_residue_atoms_count.argtypes = [POINTER(CHFL_RESIDUE), POINTER(c_uint64)]
    c_lib.chfl_residue_atoms_count.restype = chfl_status
    c_lib.chfl_residue_atoms_count.errcheck = _check_return_code

    # Function "chfl_residue_atoms", at residue.h:123
    c_lib.chfl_residue_atoms.argtypes = [POINTER(CHFL_RESIDUE), ndpointer(np.uint64, flags="C_CONTIGUOUS", ndim=1), c_uint64]
    c_lib.chfl_residue_atoms.restype = chfl_status
    c_lib.chfl_residue_atoms.errcheck = _check_return_code

    # Function "chfl_residue_id", at residue.h:136
    c_lib.chfl_residue_id.argtypes = [POINTER(CHFL_RESIDUE), POINTER(c_int64)]
    c_lib.chfl_residue_id.restype = chfl_status
    c_lib.chfl_residue_id.errcheck = _check_return_code

    # Function "chfl_residue_name", at residue.h:148
    c_lib.chfl_residue_name.argtypes = [POINTER(CHFL_RESIDUE), c_char_p, c_uint64]
    c_lib.chfl_residue_name.restype = chfl_status
    c_lib.chfl_residue_name.errcheck = _check_return_code

    # Function "chfl_residue_add_atom", at residue.h:157
    c_lib.chfl_residue_add_atom.argtypes = [POINTER(CHFL_RESIDUE), c_uint64]
    c_lib.chfl_residue_add_atom.restype = chfl_status
    c_lib.chfl_residue_add_atom.errcheck = _check_return_code

    # Function "chfl_residue_contains", at residue.h:167
    c_lib.chfl_residue_contains.argtypes = [POINTER(CHFL_RESIDUE), c_uint64, POINTER(c_bool)]
    c_lib.chfl_residue_contains.restype = chfl_status
    c_lib.chfl_residue_contains.errcheck = _check_return_code

    # Function "chfl_residue_properties_count", at residue.h:176
    c_lib.chfl_residue_properties_count.argtypes = [POINTER(CHFL_RESIDUE), POINTER(c_uint64)]
    c_lib.chfl_residue_properties_count.restype = chfl_status
    c_lib.chfl_residue_properties_count.errcheck = _check_return_code

    # Function "chfl_residue_list_properties", at residue.h:192
    c_lib.chfl_residue_list_properties.argtypes = [POINTER(CHFL_RESIDUE), POINTER(c_char_p), c_uint64]
    c_lib.chfl_residue_list_properties.restype = chfl_status
    c_lib.chfl_residue_list_properties.errcheck = _check_return_code

    # Function "chfl_residue_set_property", at residue.h:204
    c_lib.chfl_residue_set_property.argtypes = [POINTER(CHFL_RESIDUE), c_char_p, POINTER(CHFL_PROPERTY)]
    c_lib.chfl_residue_set_property.restype = chfl_status
    c_lib.chfl_residue_set_property.errcheck = _check_return_code

    # Function "chfl_residue_get_property", at residue.h:218
    c_lib.chfl_residue_get_property.argtypes = [POINTER(CHFL_RESIDUE), c_char_p]
    c_lib.chfl_residue_get_property.restype = POINTER(CHFL_PROPERTY)

    # Function "chfl_topology", at topology.h:25
    c_lib.chfl_topology.argtypes = []
    c_lib.chfl_topology.restype = POINTER(CHFL_TOPOLOGY)

    # Function "chfl_topology_from_frame", at topology.h:38
    c_lib.chfl_topology_from_frame.argtypes = [POINTER(CHFL_FRAME)]
    c_lib.chfl_topology_from_frame.restype = POINTER(CHFL_TOPOLOGY)

    # Function "chfl_topology_copy", at topology.h:48
    c_lib.chfl_topology_copy.argtypes = [POINTER(CHFL_TOPOLOGY)]
    c_lib.chfl_topology_copy.restype = POINTER(CHFL_TOPOLOGY)

    # Function "chfl_topology_atoms_count", at topology.h:56
    c_lib.chfl_topology_atoms_count.argtypes = [POINTER(CHFL_TOPOLOGY), POINTER(c_uint64)]
    c_lib.chfl_topology_atoms_count.restype = chfl_status
    c_lib.chfl_topology_atoms_count.errcheck = _check_return_code

    # Function "chfl_topology_resize", at topology.h:68
    c_lib.chfl_topology_resize.argtypes = [POINTER(CHFL_TOPOLOGY), c_uint64]
    c_lib.chfl_topology_resize.restype = chfl_status
    c_lib.chfl_topology_resize.errcheck = _check_return_code

    # Function "chfl_topology_add_atom", at topology.h:75
    c_lib.chfl_topology_add_atom.argtypes = [POINTER(CHFL_TOPOLOGY), POINTER(CHFL_ATOM)]
    c_lib.chfl_topology_add_atom.restype = chfl_status
    c_lib.chfl_topology_add_atom.errcheck = _check_return_code

    # Function "chfl_topology_remove", at topology.h:86
    c_lib.chfl_topology_remove.argtypes = [POINTER(CHFL_TOPOLOGY), c_uint64]
    c_lib.chfl_topology_remove.restype = chfl_status
    c_lib.chfl_topology_remove.errcheck = _check_return_code

    # Function "chfl_topology_bonds_count", at topology.h:95
    c_lib.chfl_topology_bonds_count.argtypes = [POINTER(CHFL_TOPOLOGY), POINTER(c_uint64)]
    c_lib.chfl_topology_bonds_count.restype = chfl_status
    c_lib.chfl_topology_bonds_count.errcheck = _check_return_code

    # Function "chfl_topology_angles_count", at topology.h:104
    c_lib.chfl_topology_angles_count.argtypes = [POINTER(CHFL_TOPOLOGY), POINTER(c_uint64)]
    c_lib.chfl_topology_angles_count.restype = chfl_status
    c_lib.chfl_topology_angles_count.errcheck = _check_return_code

    # Function "chfl_topology_dihedrals_count", at topology.h:113
    c_lib.chfl_topology_dihedrals_count.argtypes = [POINTER(CHFL_TOPOLOGY), POINTER(c_uint64)]
    c_lib.chfl_topology_dihedrals_count.restype = chfl_status
    c_lib.chfl_topology_dihedrals_count.errcheck = _check_return_code

    # Function "chfl_topology_impropers_count", at topology.h:122
    c_lib.chfl_topology_impropers_count.argtypes = [POINTER(CHFL_TOPOLOGY), POINTER(c_uint64)]
    c_lib.chfl_topology_impropers_count.restype = chfl_status
    c_lib.chfl_topology_impropers_count.errcheck = _check_return_code

    # Function "chfl_topology_bonds", at topology.h:135
    c_lib.chfl_topology_bonds.argtypes = [POINTER(CHFL_TOPOLOGY), ndpointer(np.uint64, flags="C_CONTIGUOUS", ndim=2), c_uint64]
    c_lib.chfl_topology_bonds.restype = chfl_status
    c_lib.chfl_topology_bonds.errcheck = _check_return_code

    # Function "chfl_topology_angles", at topology.h:148
    c_lib.chfl_topology_angles.argtypes = [POINTER(CHFL_TOPOLOGY), ndpointer(np.uint64, flags="C_CONTIGUOUS", ndim=2), c_uint64]
    c_lib.chfl_topology_angles.restype = chfl_status
    c_lib.chfl_topology_angles.errcheck = _check_return_code

    # Function "chfl_topology_dihedrals", at topology.h:162
    c_lib.chfl_topology_dihedrals.argtypes = [POINTER(CHFL_TOPOLOGY), ndpointer(np.uint64, flags="C_CONTIGUOUS", ndim=2), c_uint64]
    c_lib.chfl_topology_dihedrals.restype = chfl_status
    c_lib.chfl_topology_dihedrals.errcheck = _check_return_code

    # Function "chfl_topology_impropers", at topology.h:176
    c_lib.chfl_topology_impropers.argtypes = [POINTER(CHFL_TOPOLOGY), ndpointer(np.uint64, flags="C_CONTIGUOUS", ndim=2), c_uint64]
    c_lib.chfl_topology_impropers.restype = chfl_status
    c_lib.chfl_topology_impropers.errcheck = _check_return_code

    # Function "chfl_topology_add_bond", at topology.h:185
    c_lib.chfl_topology_add_bond.argtypes = [POINTER(CHFL_TOPOLOGY), c_uint64, c_uint64]
    c_lib.chfl_topology_add_bond.restype = chfl_status
    c_lib.chfl_topology_add_bond.errcheck = _check_return_code

    # Function "chfl_topology_remove_bond", at topology.h:197
    c_lib.chfl_topology_remove_bond.argtypes = [POINTER(CHFL_TOPOLOGY), c_uint64, c_uint64]
    c_lib.chfl_topology_remove_bond.restype = chfl_status
    c_lib.chfl_topology_remove_bond.errcheck = _check_return_code

    # Function "chfl_topology_clear_bonds", at topology.h:207
    c_lib.chfl_topology_clear_bonds.argtypes = [POINTER(CHFL_TOPOLOGY)]
    c_lib.chfl_topology_clear_bonds.restype = chfl_status
    c_lib.chfl_topology_clear_bonds.errcheck = _check_return_code

    # Function "chfl_topology_residues_count", at topology.h:215
    c_lib.chfl_topology_residues_count.argtypes = [POINTER(CHFL_TOPOLOGY), POINTER(c_uint64)]
    c_lib.chfl_topology_residues_count.restype = chfl_status
    c_lib.chfl_topology_residues_count.errcheck = _check_return_code

    # Function "chfl_topology_add_residue", at topology.h:227
    c_lib.chfl_topology_add_residue.argtypes = [POINTER(CHFL_TOPOLOGY), POINTER(CHFL_RESIDUE)]
    c_lib.chfl_topology_add_residue.restype = chfl_status
    c_lib.chfl_topology_add_residue.errcheck = _check_return_code

    # Function "chfl_topology_residues_linked", at topology.h:238
    c_lib.chfl_topology_residues_linked.argtypes = [POINTER(CHFL_TOPOLOGY), POINTER(CHFL_RESIDUE), POINTER(CHFL_RESIDUE), POINTER(c_bool)]
    c_lib.chfl_topology_residues_linked.restype = chfl_status
    c_lib.chfl_topology_residues_linked.errcheck = _check_return_code

    # Function "chfl_topology_bond_with_order", at topology.h:251
    c_lib.chfl_topology_bond_with_order.argtypes = [POINTER(CHFL_TOPOLOGY), c_uint64, c_uint64, chfl_bond_order]
    c_lib.chfl_topology_bond_with_order.restype = chfl_status
    c_lib.chfl_topology_bond_with_order.errcheck = _check_return_code

    # Function "chfl_topology_bond_orders", at topology.h:265
    c_lib.chfl_topology_bond_orders.argtypes = [POINTER(CHFL_TOPOLOGY), ndpointer(chfl_bond_order, flags="C_CONTIGUOUS", ndim=1), c_uint64]
    c_lib.chfl_topology_bond_orders.restype = chfl_status
    c_lib.chfl_topology_bond_orders.errcheck = _check_return_code

    # Function "chfl_topology_bond_order", at topology.h:278
    c_lib.chfl_topology_bond_order.argtypes = [POINTER(CHFL_TOPOLOGY), c_uint64, c_uint64, POINTER(chfl_bond_order)]
    c_lib.chfl_topology_bond_order.restype = chfl_status
    c_lib.chfl_topology_bond_order.errcheck = _check_return_code

    # Function "chfl_cell", at cell.h:40
    c_lib.chfl_cell.argtypes = [chfl_vector3d, chfl_vector3d]
    c_lib.chfl_cell.restype = POINTER(CHFL_CELL)

    # Function "chfl_cell_from_matrix", at cell.h:55
    c_lib.chfl_cell_from_matrix.argtypes = [ARRAY(chfl_vector3d, (3))]
    c_lib.chfl_cell_from_matrix.restype = POINTER(CHFL_CELL)

    # Function "chfl_cell_from_frame", at cell.h:68
    c_lib.chfl_cell_from_frame.argtypes = [POINTER(CHFL_FRAME)]
    c_lib.chfl_cell_from_frame.restype = POINTER(CHFL_CELL)

    # Function "chfl_cell_copy", at cell.h:78
    c_lib.chfl_cell_copy.argtypes = [POINTER(CHFL_CELL)]
    c_lib.chfl_cell_copy.restype = POINTER(CHFL_CELL)

    # Function "chfl_cell_volume", at cell.h:85
    c_lib.chfl_cell_volume.argtypes = [POINTER(CHFL_CELL), POINTER(c_double)]
    c_lib.chfl_cell_volume.restype = chfl_status
    c_lib.chfl_cell_volume.errcheck = _check_return_code

    # Function "chfl_cell_lengths", at cell.h:94
    c_lib.chfl_cell_lengths.argtypes = [POINTER(CHFL_CELL), chfl_vector3d]
    c_lib.chfl_cell_lengths.restype = chfl_status
    c_lib.chfl_cell_lengths.errcheck = _check_return_code

    # Function "chfl_cell_set_lengths", at cell.h:110
    c_lib.chfl_cell_set_lengths.argtypes = [POINTER(CHFL_CELL), chfl_vector3d]
    c_lib.chfl_cell_set_lengths.restype = chfl_status
    c_lib.chfl_cell_set_lengths.errcheck = _check_return_code

    # Function "chfl_cell_angles", at cell.h:119
    c_lib.chfl_cell_angles.argtypes = [POINTER(CHFL_CELL), chfl_vector3d]
    c_lib.chfl_cell_angles.restype = chfl_status
    c_lib.chfl_cell_angles.errcheck = _check_return_code

    # Function "chfl_cell_set_angles", at cell.h:137
    c_lib.chfl_cell_set_angles.argtypes = [POINTER(CHFL_CELL), chfl_vector3d]
    c_lib.chfl_cell_set_angles.restype = chfl_status
    c_lib.chfl_cell_set_angles.errcheck = _check_return_code

    # Function "chfl_cell_matrix", at cell.h:146
    c_lib.chfl_cell_matrix.argtypes = [POINTER(CHFL_CELL), ARRAY(chfl_vector3d, (3))]
    c_lib.chfl_cell_matrix.restype = chfl_status
    c_lib.chfl_cell_matrix.errcheck = _check_return_code

    # Function "chfl_cell_shape", at cell.h:155
    c_lib.chfl_cell_shape.argtypes = [POINTER(CHFL_CELL), POINTER(chfl_cellshape)]
    c_lib.chfl_cell_shape.restype = chfl_status
    c_lib.chfl_cell_shape.errcheck = _check_return_code

    # Function "chfl_cell_set_shape", at cell.h:164
    c_lib.chfl_cell_set_shape.argtypes = [POINTER(CHFL_CELL), chfl_cellshape]
    c_lib.chfl_cell_set_shape.restype = chfl_status
    c_lib.chfl_cell_set_shape.errcheck = _check_return_code

    # Function "chfl_cell_wrap", at cell.h:173
    c_lib.chfl_cell_wrap.argtypes = [POINTER(CHFL_CELL), chfl_vector3d]
    c_lib.chfl_cell_wrap.restype = chfl_status
    c_lib.chfl_cell_wrap.errcheck = _check_return_code

    # Function "chfl_frame", at frame.h:25
    c_lib.chfl_frame.argtypes = []
    c_lib.chfl_frame.restype = POINTER(CHFL_FRAME)

    # Function "chfl_frame_copy", at frame.h:35
    c_lib.chfl_frame_copy.argtypes = [POINTER(CHFL_FRAME)]
    c_lib.chfl_frame_copy.restype = POINTER(CHFL_FRAME)

    # Function "chfl_frame_atoms_count", at frame.h:43
    c_lib.chfl_frame_atoms_count.argtypes = [POINTER(CHFL_FRAME), POINTER(c_uint64)]
    c_lib.chfl_frame_atoms_count.restype = chfl_status
    c_lib.chfl_frame_atoms_count.errcheck = _check_return_code

    # Function "chfl_frame_positions", at frame.h:62
    c_lib.chfl_frame_positions.argtypes = [POINTER(CHFL_FRAME), POINTER(POINTER(chfl_vector3d)), POINTER(c_uint64)]
    c_lib.chfl_frame_positions.restype = chfl_status
    c_lib.chfl_frame_positions.errcheck = _check_return_code

    # Function "chfl_frame_velocities", at frame.h:85
    c_lib.chfl_frame_velocities.argtypes = [POINTER(CHFL_FRAME), POINTER(POINTER(chfl_vector3d)), POINTER(c_uint64)]
    c_lib.chfl_frame_velocities.restype = chfl_status
    c_lib.chfl_frame_velocities.errcheck = _check_return_code

    # Function "chfl_frame_add_atom", at frame.h:97
    c_lib.chfl_frame_add_atom.argtypes = [POINTER(CHFL_FRAME), POINTER(CHFL_ATOM), chfl_vector3d, chfl_vector3d]
    c_lib.chfl_frame_add_atom.restype = chfl_status
    c_lib.chfl_frame_add_atom.errcheck = _check_return_code

    # Function "chfl_frame_remove", at frame.h:110
    c_lib.chfl_frame_remove.argtypes = [POINTER(CHFL_FRAME), c_uint64]
    c_lib.chfl_frame_remove.restype = chfl_status
    c_lib.chfl_frame_remove.errcheck = _check_return_code

    # Function "chfl_frame_resize", at frame.h:122
    c_lib.chfl_frame_resize.argtypes = [POINTER(CHFL_FRAME), c_uint64]
    c_lib.chfl_frame_resize.restype = chfl_status
    c_lib.chfl_frame_resize.errcheck = _check_return_code

    # Function "chfl_frame_add_velocities", at frame.h:134
    c_lib.chfl_frame_add_velocities.argtypes = [POINTER(CHFL_FRAME)]
    c_lib.chfl_frame_add_velocities.restype = chfl_status
    c_lib.chfl_frame_add_velocities.errcheck = _check_return_code

    # Function "chfl_frame_has_velocities", at frame.h:142
    c_lib.chfl_frame_has_velocities.argtypes = [POINTER(CHFL_FRAME), POINTER(c_bool)]
    c_lib.chfl_frame_has_velocities.restype = chfl_status
    c_lib.chfl_frame_has_velocities.errcheck = _check_return_code

    # Function "chfl_frame_set_cell", at frame.h:151
    c_lib.chfl_frame_set_cell.argtypes = [POINTER(CHFL_FRAME), POINTER(CHFL_CELL)]
    c_lib.chfl_frame_set_cell.restype = chfl_status
    c_lib.chfl_frame_set_cell.errcheck = _check_return_code

    # Function "chfl_frame_set_topology", at frame.h:163
    c_lib.chfl_frame_set_topology.argtypes = [POINTER(CHFL_FRAME), POINTER(CHFL_TOPOLOGY)]
    c_lib.chfl_frame_set_topology.restype = chfl_status
    c_lib.chfl_frame_set_topology.errcheck = _check_return_code

    # Function "chfl_frame_step", at frame.h:173
    c_lib.chfl_frame_step.argtypes = [POINTER(CHFL_FRAME), POINTER(c_uint64)]
    c_lib.chfl_frame_step.restype = chfl_status
    c_lib.chfl_frame_step.errcheck = _check_return_code

    # Function "chfl_frame_set_step", at frame.h:182
    c_lib.chfl_frame_set_step.argtypes = [POINTER(CHFL_FRAME), c_uint64]
    c_lib.chfl_frame_set_step.restype = chfl_status
    c_lib.chfl_frame_set_step.errcheck = _check_return_code

    # Function "chfl_frame_guess_bonds", at frame.h:194
    c_lib.chfl_frame_guess_bonds.argtypes = [POINTER(CHFL_FRAME)]
    c_lib.chfl_frame_guess_bonds.restype = chfl_status
    c_lib.chfl_frame_guess_bonds.errcheck = _check_return_code

    # Function "chfl_frame_distance", at frame.h:203
    c_lib.chfl_frame_distance.argtypes = [POINTER(CHFL_FRAME), c_uint64, c_uint64, POINTER(c_double)]
    c_lib.chfl_frame_distance.restype = chfl_status
    c_lib.chfl_frame_distance.errcheck = _check_return_code

    # Function "chfl_frame_angle", at frame.h:214
    c_lib.chfl_frame_angle.argtypes = [POINTER(CHFL_FRAME), c_uint64, c_uint64, c_uint64, POINTER(c_double)]
    c_lib.chfl_frame_angle.restype = chfl_status
    c_lib.chfl_frame_angle.errcheck = _check_return_code

    # Function "chfl_frame_dihedral", at frame.h:225
    c_lib.chfl_frame_dihedral.argtypes = [POINTER(CHFL_FRAME), c_uint64, c_uint64, c_uint64, c_uint64, POINTER(c_double)]
    c_lib.chfl_frame_dihedral.restype = chfl_status
    c_lib.chfl_frame_dihedral.errcheck = _check_return_code

    # Function "chfl_frame_out_of_plane", at frame.h:239
    c_lib.chfl_frame_out_of_plane.argtypes = [POINTER(CHFL_FRAME), c_uint64, c_uint64, c_uint64, c_uint64, POINTER(c_double)]
    c_lib.chfl_frame_out_of_plane.restype = chfl_status
    c_lib.chfl_frame_out_of_plane.errcheck = _check_return_code

    # Function "chfl_frame_properties_count", at frame.h:248
    c_lib.chfl_frame_properties_count.argtypes = [POINTER(CHFL_FRAME), POINTER(c_uint64)]
    c_lib.chfl_frame_properties_count.restype = chfl_status
    c_lib.chfl_frame_properties_count.errcheck = _check_return_code

    # Function "chfl_frame_list_properties", at frame.h:264
    c_lib.chfl_frame_list_properties.argtypes = [POINTER(CHFL_FRAME), POINTER(c_char_p), c_uint64]
    c_lib.chfl_frame_list_properties.restype = chfl_status
    c_lib.chfl_frame_list_properties.errcheck = _check_return_code

    # Function "chfl_frame_set_property", at frame.h:276
    c_lib.chfl_frame_set_property.argtypes = [POINTER(CHFL_FRAME), c_char_p, POINTER(CHFL_PROPERTY)]
    c_lib.chfl_frame_set_property.restype = chfl_status
    c_lib.chfl_frame_set_property.errcheck = _check_return_code

    # Function "chfl_frame_get_property", at frame.h:290
    c_lib.chfl_frame_get_property.argtypes = [POINTER(CHFL_FRAME), c_char_p]
    c_lib.chfl_frame_get_property.restype = POINTER(CHFL_PROPERTY)

    # Function "chfl_frame_add_bond", at frame.h:299
    c_lib.chfl_frame_add_bond.argtypes = [POINTER(CHFL_FRAME), c_uint64, c_uint64]
    c_lib.chfl_frame_add_bond.restype = chfl_status
    c_lib.chfl_frame_add_bond.errcheck = _check_return_code

    # Function "chfl_frame_bond_with_order", at frame.h:309
    c_lib.chfl_frame_bond_with_order.argtypes = [POINTER(CHFL_FRAME), c_uint64, c_uint64, chfl_bond_order]
    c_lib.chfl_frame_bond_with_order.restype = chfl_status
    c_lib.chfl_frame_bond_with_order.errcheck = _check_return_code

    # Function "chfl_frame_remove_bond", at frame.h:321
    c_lib.chfl_frame_remove_bond.argtypes = [POINTER(CHFL_FRAME), c_uint64, c_uint64]
    c_lib.chfl_frame_remove_bond.restype = chfl_status
    c_lib.chfl_frame_remove_bond.errcheck = _check_return_code

    # Function "chfl_frame_clear_bonds", at frame.h:331
    c_lib.chfl_frame_clear_bonds.argtypes = [POINTER(CHFL_FRAME)]
    c_lib.chfl_frame_clear_bonds.restype = chfl_status
    c_lib.chfl_frame_clear_bonds.errcheck = _check_return_code

    # Function "chfl_frame_add_residue", at frame.h:341
    c_lib.chfl_frame_add_residue.argtypes = [POINTER(CHFL_FRAME), POINTER(CHFL_RESIDUE)]
    c_lib.chfl_frame_add_residue.restype = chfl_status
    c_lib.chfl_frame_add_residue.errcheck = _check_return_code

    # Function "chfl_trajectory_open", at trajectory.h:26
    c_lib.chfl_trajectory_open.argtypes = [c_char_p, c_char]
    c_lib.chfl_trajectory_open.restype = POINTER(CHFL_TRAJECTORY)

    # Function "chfl_trajectory_with_format", at trajectory.h:43
    c_lib.chfl_trajectory_with_format.argtypes = [c_char_p, c_char, c_char_p]
    c_lib.chfl_trajectory_with_format.restype = POINTER(CHFL_TRAJECTORY)

    # Function "chfl_trajectory_memory_reader", at trajectory.h:59
    c_lib.chfl_trajectory_memory_reader.argtypes = [c_char_p, c_uint64, c_char_p]
    c_lib.chfl_trajectory_memory_reader.restype = POINTER(CHFL_TRAJECTORY)

    # Function "chfl_trajectory_memory_writer", at trajectory.h:74
    c_lib.chfl_trajectory_memory_writer.argtypes = [c_char_p]
    c_lib.chfl_trajectory_memory_writer.restype = POINTER(CHFL_TRAJECTORY)

    # Function "chfl_trajectory_path", at trajectory.h:86
    c_lib.chfl_trajectory_path.argtypes = [POINTER(CHFL_TRAJECTORY), c_char_p, c_uint64]
    c_lib.chfl_trajectory_path.restype = chfl_status
    c_lib.chfl_trajectory_path.errcheck = _check_return_code

    # Function "chfl_trajectory_read", at trajectory.h:98
    c_lib.chfl_trajectory_read.argtypes = [POINTER(CHFL_TRAJECTORY), POINTER(CHFL_FRAME)]
    c_lib.chfl_trajectory_read.restype = chfl_status
    c_lib.chfl_trajectory_read.errcheck = _check_return_code

    # Function "chfl_trajectory_read_step", at trajectory.h:110
    c_lib.chfl_trajectory_read_step.argtypes = [POINTER(CHFL_TRAJECTORY), c_uint64, POINTER(CHFL_FRAME)]
    c_lib.chfl_trajectory_read_step.restype = chfl_status
    c_lib.chfl_trajectory_read_step.errcheck = _check_return_code

    # Function "chfl_trajectory_write", at trajectory.h:119
    c_lib.chfl_trajectory_write.argtypes = [POINTER(CHFL_TRAJECTORY), POINTER(CHFL_FRAME)]
    c_lib.chfl_trajectory_write.restype = chfl_status
    c_lib.chfl_trajectory_write.errcheck = _check_return_code

    # Function "chfl_trajectory_set_topology", at trajectory.h:130
    c_lib.chfl_trajectory_set_topology.argtypes = [POINTER(CHFL_TRAJECTORY), POINTER(CHFL_TOPOLOGY)]
    c_lib.chfl_trajectory_set_topology.restype = chfl_status
    c_lib.chfl_trajectory_set_topology.errcheck = _check_return_code

    # Function "chfl_trajectory_topology_file", at trajectory.h:144
    c_lib.chfl_trajectory_topology_file.argtypes = [POINTER(CHFL_TRAJECTORY), c_char_p, c_char_p]
    c_lib.chfl_trajectory_topology_file.restype = chfl_status
    c_lib.chfl_trajectory_topology_file.errcheck = _check_return_code

    # Function "chfl_trajectory_set_cell", at trajectory.h:154
    c_lib.chfl_trajectory_set_cell.argtypes = [POINTER(CHFL_TRAJECTORY), POINTER(CHFL_CELL)]
    c_lib.chfl_trajectory_set_cell.restype = chfl_status
    c_lib.chfl_trajectory_set_cell.errcheck = _check_return_code

    # Function "chfl_trajectory_nsteps", at trajectory.h:164
    c_lib.chfl_trajectory_nsteps.argtypes = [POINTER(CHFL_TRAJECTORY), POINTER(c_uint64)]
    c_lib.chfl_trajectory_nsteps.restype = chfl_status
    c_lib.chfl_trajectory_nsteps.errcheck = _check_return_code

    # Function "chfl_trajectory_memory_buffer", at trajectory.h:178
    c_lib.chfl_trajectory_memory_buffer.argtypes = [POINTER(CHFL_TRAJECTORY), POINTER(c_char_p), POINTER(c_uint64)]
    c_lib.chfl_trajectory_memory_buffer.restype = chfl_status
    c_lib.chfl_trajectory_memory_buffer.errcheck = _check_return_code

    # Function "chfl_selection", at selection.h:24
    c_lib.chfl_selection.argtypes = [c_char_p]
    c_lib.chfl_selection.restype = POINTER(CHFL_SELECTION)

    # Function "chfl_selection_copy", at selection.h:37
    c_lib.chfl_selection_copy.argtypes = [POINTER(CHFL_SELECTION)]
    c_lib.chfl_selection_copy.restype = POINTER(CHFL_SELECTION)

    # Function "chfl_selection_size", at selection.h:49
    c_lib.chfl_selection_size.argtypes = [POINTER(CHFL_SELECTION), POINTER(c_uint64)]
    c_lib.chfl_selection_size.restype = chfl_status
    c_lib.chfl_selection_size.errcheck = _check_return_code

    # Function "chfl_selection_string", at selection.h:62
    c_lib.chfl_selection_string.argtypes = [POINTER(CHFL_SELECTION), c_char_p, c_uint64]
    c_lib.chfl_selection_string.restype = chfl_status
    c_lib.chfl_selection_string.errcheck = _check_return_code

    # Function "chfl_selection_evaluate", at selection.h:75
    c_lib.chfl_selection_evaluate.argtypes = [POINTER(CHFL_SELECTION), POINTER(CHFL_FRAME), POINTER(c_uint64)]
    c_lib.chfl_selection_evaluate.restype = chfl_status
    c_lib.chfl_selection_evaluate.errcheck = _check_return_code

    # Function "chfl_selection_matches", at selection.h:87
    c_lib.chfl_selection_matches.argtypes = [POINTER(CHFL_SELECTION), ndpointer(chfl_match, flags="C_CONTIGUOUS", ndim=1), c_uint64]
    c_lib.chfl_selection_matches.restype = chfl_status
    c_lib.chfl_selection_matches.errcheck = _check_return_code
